// DOMTest.java
// A sample code for java2 jaxp from XML SQUARE at unitec:
// http://www.utj.co.jp/xml/dev/java/dxjava_1.html

import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
public class DOMview {
    private static Document gDocument = null;
    private static Element gAttrNode = null;

    public static void main (String[] args) {
        try {
            DocumentBuilderFactory dbfactory
		= DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = dbfactory.newDocumentBuilder();
            Document d = gDocument = builder.parse(args[0]);

	    //Main
	    Element root = d.getDocumentElement();
	    removeWhiteSpaceNodes(root);
	    printTree(root, d);
	    
            System.out.println("done.");
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    private static void printTree(Node node, Document doc) {
	Stack path = new Stack(); //Stack of nodes
	printTree0(node, path);
    }

    private static void printTree0(Node node, Stack path) {
	path.push(node);

	//Printing the element
	printPath(path);
	printNode(node);

	//Printing its attribute children
	printAtributeNodes(node, path);

	//Printing its element children
	Node child= node.getFirstChild();
	while (child!= null) {
	    printTree0(child, path);
	    child= child.getNextSibling();
	}
	path.pop();
    }

    static Element getDammyAttrNode() {
	if (gAttrNode == null) {
	    if (gDocument == null) {
		System.out.println("no document allocated!");
		System.exit(1);
	    }
	    gAttrNode = gDocument.createElement("dammyAttribute");
	}
	return gAttrNode;
    }

    public static void printAtributeNodes(Node node, Stack path) {
	if (node.getNodeType() != Node.ELEMENT_NODE)
	    return;

	//Get a set of attributes
	Element elem = (Element) node; //Downward type casting
	NamedNodeMap map = elem.getAttributes();
	ArrayList attrnameList = new ArrayList();

	//Store attribute nodes into a list;
	for (int i = 0; i < map.getLength(); i++) {
	    Attr attr = (Attr) map.item(i);
	    String attrname = attr.getNodeName();	//Get attribute name 
	    String attrvalue = map.getNamedItem(attrname).getNodeValue();

	    //Printing attribute name and value
	    path.push(getDammyAttrNode());
	    printPath(path);
	    System.out.println("@"+attrname+"=\""+attrvalue.trim()+"\"");
	    path.pop();
	}
    }
    private static void printPath(Stack path) {
	int len = path.size();
	for (int i = 0; i < len; i++) {
	    Node parent = (Node) path.get(i);
	    if (i < len-1) {
		if (parent.getNextSibling() == null) { //rightmost
 		    System.out.print("   "); //rightmost
		} else {
 		    System.out.print("|  ");
		}
	    }
	    else {
		if (parent.getNextSibling() == null) { //rightmost
		    System.out.print("`--"); //rightmost
		} else {
 		    System.out.print("|--");
		}
	    }
	}
    }

    private static void printNode(Node node) {
	switch (node.getNodeType()) {
	case Node.ELEMENT_NODE: {
	    System.out.println(""+node.getNodeName());
	    break;
	}
	case Node.TEXT_NODE: {
	    System.out.println("\""+normalizeText(node.getNodeValue()).trim()+"\"");
	    break;
	}
	case Node.PROCESSING_INSTRUCTION_NODE: {
	    System.out.println("\""+normalizeText(node.getNodeValue())+"\"");
	    break;
	}
	case Node.COMMENT_NODE: {
	    System.out.println("\""+normalizeText(node.getNodeValue())+"\"");
	    break;
	}
	default: {
	    System.out.println("Other nodetype: "+node.getNodeType());
	    break;
	}
        }//switch
    }

    static String normalizeText(String text) {
	StringBuffer buf = new StringBuffer();
	for (int i = 0; i < text.length(); i++) {
	    char C = text.charAt(i);
	    switch (C) {
	    case '\t': case ' ':
		{
		    buf.append(' ');
		    break;
		}
	    case '\n': case '\r': case '\f': case '\b':
		{
		    buf.append("\\n");
		    break;
		}
	    default:
		{
		    buf.append(C);
		    break;
		}
	    }
	}
	return buf.toString();
    }

    private static void removeWhiteSpaceNodes(Node node) {
	Node nextnode= node.getFirstChild();
	while (nextnode!= null) {
	    Node child = nextnode;
	    nextnode= child.getNextSibling();

	    if (child.getNodeType() == Node.TEXT_NODE) {
		//For a text node
		if (isWhitespaceString(child.getNodeValue())) {
		    //Remove ignorable whitespace
		    node.removeChild(child);
		}
	    }
	    else if (child.getNodeType() == Node.ELEMENT_NODE) {
		//For an element node
		removeWhiteSpaceNodes(child);
	    }
	}
    }

    private static boolean isWhitespaceString(String text) {
	for (int i = 0; i < text.length(); i++) {
	    if (!Character.isWhitespace(text.charAt(i)))
		return false;
	}
	return true;
    }

}

class Stack extends ArrayList {
    void push(Object o) {
	this.add(o);
    }

    Object pop() {
	int last = this.size() - 1;
	if (last < 0) {
	    System.out.println("Stack: cannot pop the empty stack!");
	    System.exit(1);
	}
	Object o = this.get(last);
	this.remove(last);
	return o;
    }
}
