/* Search.java
   Copyright 2002, 
   Tatsuya Asai, Kenji Abe, Shinji Kawasoe, Hiroshi Sakamoto,
   Hiroki Arimura and Setsuo Arikawa,
   all rights reserved.

   This software may be freely used for academic purpose only. Please
   contact us for profit use. However, when distributed, the original
   source must be clearly stated, and, when the source code is
   distributed, the copiright notice must be retained and any
   alterations in the code must be clearly marked. No warranty is
   given regarding the quality of this software.
*/

import java.io.*;
import java.lang.*;
import java.util.*;
import org.w3c.dom.*;

class Search {

    /**************************************************
     * Mining Subroutines
     **************************************************/

    public static void
    execute(Node domRoot) throws Exception{
	PatternList plist;

	plist = mineFirstStage(domRoot);
	Freqt.gTotalNumFreq += plist.size();
	Freqt.gStage++;

	while (plist.size() > 0){
	    if (Freqt.gStage > Freqt.gMaxStage && Freqt.gMaxStage > 0) return;
	    plist = mineMainStage(plist);
	    Freqt.gTotalNumFreq += plist.size();
	    Freqt.gStage++;
	}
	
    }

    /**************************************************
     * Mining Subroutines
     **************************************************/

    static PatternList mineFirstStage(Node domRoot){

        PatternList newPatList = new PatternList();

	Freqt.gTotalTimer.start();
	CpuTimer stageTimer = new CpuTimer(CpuTimer.ON);

	//Computes the patterns of size one
	HashMap patHashMap = new HashMap();
	Freqt.scanDomTree(domRoot, patHashMap);
	newPatList.addAll(patHashMap.values());
	Freqt.computeRootOccFirstStage(newPatList);

	newPatList = Freqt.pruneFirstStage(newPatList);
	
	stageTimer.stop();
	Freqt.gTotalTimer.stop();

	Freqt.printFreqPatterns(newPatList, Freqt.gStage);

	stageTimer.print("@stage "+Freqt.gStage+": @time \t");
        return newPatList;
    }

    static PatternList mineMainStage(PatternList plist){
	if (Freqt.DEBUG_TRACE) System.out.println("@debug: Enter stage"+Freqt.gStage);
        PatternList newplist = new PatternList();

	Freqt.gTotalTimer.start();
	CpuTimer stageTimer = new CpuTimer(CpuTimer.ON);

	//Expand a treePattern and update its rightmost occurrences (RMOs).
        for (int i = 0; i < plist.size(); i++){
            Pattern pattern = (Pattern)plist.get(i);
	    newplist.addAll(expandPattern(pattern));
	}

	//Pre-pruning
	newplist = Freqt.pruneByRMO(newplist);

	//Count the root occurrence of the pattern
	Freqt.computeRootOcc(newplist); 
	//setocc(newplist);

	//Pruning
        newplist = Freqt.pruneMainStage(newplist);

	stageTimer.stop();
	Freqt.gTotalTimer.stop();

	Freqt.printFreqPatterns(newplist, Freqt.gStage);

	stageTimer.print("@stage "+Freqt.gStage+": @time \t");

	if (Freqt.DEBUG_TRACE) System.out.println("@debug: Exit stage"+Freqt.gStage);
        return newplist;
    }

    /** Compute the set of the rightmost expansions of a given pattern
     *  and the updated list of their rightmost occurrences (RMOs).
     */
    static PatternList expandPattern(Pattern pattern) {
	int pmax = pattern.getRmbLength();
	PatternList successorsList = new PatternList();

	//Node:
	// - The number p is the depth of the branching node; not the 
	//   depth of <depth, label>-expansion.
	// - The depth of the newly attached node is -1 <= (p-1) <= rmb-1.
	Freqt.doRightmostExpansion(pattern, successorsList, 0);
	//doRightmostExpansionAtBottom(pattern, successorsList, 0);
	for (int p = 1; p <= pmax; p++) {
	    if (Freqt.ENABLE_DUPLICATE_DETECTION){
		Freqt.doRightmostExpansion(pattern, successorsList, p);
	    }
	    else {
		Util.errorExit("Option -nodd is not supported!");
		//doRightmostExpansionWithNoDD(pattern, successorsList, p);
	    }
	}
	return successorsList;
    }

    static boolean isNewLabelId(int nid, PatternList newPatList) {
	return (nid == newPatList.size());
    }

}

//EOF
